/* Blow Up the Frost Elf Kingdom
by Gughunter
for Game in a Day Challenge, 2007 (theme: "Frost and Flame")
9:30 AM - 3:00 PM, 2007-0922

*/


#define PULSE_SECONDS 1
#define SIGNAL_COLLAPSE_GAP 9
#define COLLAPSE_CYCLE 8
#define DYNAMITE_TIMER 12
#define ELF_COUNT 96


var/list/templates = list(
	list(1, 1, 1, 0),
	list(1, 1, 1, 0),
	list(1, 1, 0, 0),
	list(1, 1, 0, 0),
	list(1, 0, 1, 0),
	list(1, 0, 1, 0),
	list(0, 0, 0, 0)
	)
var/globalSignal
var/list/cardinalDirs = list(NORTH, SOUTH, EAST, WEST)
var/list/collapsing = list()
var/list/santas = list()
var/list/elves = list()


world
	name = "Blow Up the Frost Elf Kingdom"
	view = 8
	turf = /turf/hollow
	mob = /mob/player


	New()
		. = ..()
		spawn(1) CreateWorld()
		spawn(50) CollapseScan()


	proc/CollapseScan()
		var/collapseCycle = 0

		while(1)
			for(var/cury in 2 to world.maxy - 1)
				collapseCycle++

				if(!(collapseCycle % COLLAPSE_CYCLE))
					sleep(1)

				for(var/curx in 1 to world.maxx)
					var/turf/wall/T = locate(curx, cury, 1)
					if(istype(T)) T.Collapse()


turf
	hollow
		icon = 'hollow.dmi'


	wall
		var/lastSignal = -1
		density = 1
		icon = 'wall.dmi'


		proc/PropagateSignal()
			//useful for debugging: flick('signaltest.dmi', src)
			for(var/curDir in cardinalDirs)
				var/turf/wall/neighbor = get_step(src, curDir)
				if(istype(neighbor) && (neighbor.lastSignal < lastSignal))
					neighbor.lastSignal = lastSignal
					spawn(1) if(istype(neighbor)) neighbor.PropagateSignal()


		proc/Collapse()
			if((globalSignal - lastSignal) > SIGNAL_COLLAPSE_GAP)
				var/turf/hollow/T = get_step(src, SOUTH)
				if(istype(T))
					for(var/mob/M in T)
						M.Die()

					var/turf/wall/newWall = new /turf/wall(T)
					newWall.lastSignal = lastSignal
					new /turf/hollow(src)


		Click()
			if((usr in range(src, 1)) && (y != 1) && (!usr:dead))
				var/obj/dynamite/D = locate() in src
				if(!D)
					D = new(src)
					D.owner = usr


client
	command_text = "say "


mob/proc/Die() //stub


mob/dummy/verb/reboot()
	world.Reboot()


mob/player
	icon = 'player.dmi'
	var/score = 0
	var/dead = 0


	verb/say(T as text)
		set hidden = 1

		world << "<b>[src]</b>: [T]"


	Move()
		if(!dead) . = ..()


	Login()
		sleep(2)
		if(!loc)
			var/turf/wall/T = locate() in block(locate(1, 33, 1), locate(32, 33, 1))
			loc = get_step(T, NORTH)
		if(!(src in santas)) santas += src
		if(key == world.host)
			verbs += /mob/dummy/verb/reboot

		src << file2text("instructions.txt")


	Bump(mob/M)
		if(istype(M, /mob/elf))
			M.Die(src)


	proc/AddScore()
		world << "[src]: [++score]"


	Die()
		dead = 1
		icon_state = "corpse"

		santas -= src
		if(length(santas) == 0)
			GameOver()



proc/GameOver()
	world << "<b>GAME OVER</b>"
	world << ""
	world << "<b>Final Tallies</b>"
	world << ""
	for(var/mob/player/P in world)
		world << "[P]: [++P.score]"


mob/elf
	icon = 'frost_elf.dmi'


	Die(mob/player/killer)
		if(!(icon_state == "corpse"))
			density = 0
			icon_state = "corpse"

			if(killer) killer.AddScore()

			elves -= src
			if(length(elves) == 0)
				GameOver()


obj/dynamite
	icon = 'dynamite.dmi'
	var/mob/player/owner


	New()
		. = ..()
		spawn(0) Explode()


	proc/Explode()
		sleep(DYNAMITE_TIMER)

		icon_state = "hide"
		flick('explode.dmi', src)

		if(istype(loc, /turf/wall))
			new /turf/hollow(loc)

		for(var/mob/M in range(src, 1))
			M.Die(owner)

		spawn(10) del src



proc/CreateWorld()
	for(var/curx in 1 to 16)
		for(var/cury in 1 to 16)
			var/basex = curx * 2 - 1
			var/otherx = curx * 2
			var/basey = cury * 2 + 1
			var/othery = cury * 2

			var/list/curtemplate = pick(templates)
			var/curindex = 1
			for(var/suby in basey to othery step -1)
				for(var/subx in basex to otherx)
					if(curtemplate[curindex] == 1)
						new /turf/wall(locate(subx, suby, 1))

					++curindex

	CreateElves()


proc/CreateElves()
	var/list/L = block(locate(1, 2, 1), locate(world.maxx, 25, 1))

	for(var/i in 1 to ELF_COUNT)
		while(1)
			var/turf/hollow/T = pick(L)
			if((istype(T) && !(locate(/mob/elf) in T)))
				var/mob/elf/E = new(T)
				elves += E
				break


obj/beacon
	icon = 'beacon.dmi'


	New()
		icon_state = "hide"
		spawn Maintain()


	proc/Maintain()
		var/signal = 1
		while(1)
			loc:lastSignal = signal
			loc:PropagateSignal(signal)
			globalSignal = signal
			++signal
			sleep(PULSE_SECONDS * 10)
